package kiba300.hilfsklassen;


 /**
 *  <h4>Base64.java</h4>
 *
 *  http://stackoverflow.com/questions/469695/decode-base64-data-in-java
 *  by GeorgeK
 *  23.04.2013
 *  <br/>
 *  <p>  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  </p>
 *  <p>
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  </p>
 *  <p>
 *  You should have received a copy of the GNU General Public License
 *  along with this program (gpl.html).  If not, see http://www.gnu.org/licenses/.
 *  </p>
 */
public class Base64 
{

    public final   char[] ALPHABET = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/".toCharArray();

    public   int[]  toInt   = new int[128];

    public Base64()
    {
        for(int i=0; i< ALPHABET.length; i++){
            toInt[ALPHABET[i]]= i;
        }
    }

    /**
    Führt Base64-Wandlung für UTF8-codierte Strings durch.
    */
    public String encodeUTF8(String text)
    {
        try
        {
//###################
//TEST mit ISO-8859-15
            return encode(text.getBytes("UTF-8"));
//            return encode(text.getBytes("ISO-8859-15"));
        }
        catch(Exception e)
        {
            return null;
        }
    }

    /**
    Führt Rückwandlung einer zuvor aus einem UTF8-String gewonnenen Base64-Kodierung durch.
    */
    public String decodeUTF8(String text)
    {
        try
        {
//###################
//TEST mit ISO-8859-15
            return new String(decode(text),"UTF8");
//            return new String(decode(text),"ISO-8859-15");
        }
        catch(Exception e)
        {
            return null;
        }
    }


    /**
     * Translates the specified byte array into Base64 string.
     *
     * @param buf the byte array (not null)
     * @return the translated Base64 string (not null)
     */
    public   String encode(byte[] buf){
        int size = buf.length;
        char[] ar = new char[((size + 2) / 3) * 4];
        int a = 0;
        int i=0;
        while(i < size){
            byte b0 = buf[i++];
            byte b1 = (i < size) ? buf[i++] : 0;
            byte b2 = (i < size) ? buf[i++] : 0;

            int mask = 0x3F;
            ar[a++] = ALPHABET[(b0 >> 2) & mask];
            ar[a++] = ALPHABET[((b0 << 4) | ((b1 & 0xFF) >> 4)) & mask];
            ar[a++] = ALPHABET[((b1 << 2) | ((b2 & 0xFF) >> 6)) & mask];
            ar[a++] = ALPHABET[b2 & mask];
        }
        switch(size % 3){
            case 1: ar[--a]  = '=';
            case 2: ar[--a]  = '=';
        }
        return new String(ar);
    }

    /**
     * Translates the specified Base64 string into a byte array.
     *
     * @param s the Base64 string (not null)
     * @return the byte array (not null)
     */
    public   byte[] decode(String s){
        int delta = s.endsWith( "==" ) ? 2 : s.endsWith( "=" ) ? 1 : 0;
        byte[] buffer = new byte[s.length()*3/4 - delta];
        int mask = 0xFF;
        int index = 0;
        for(int i=0; i< s.length(); i+=4){
            int c0 = toInt[s.charAt( i )];
            int c1 = toInt[s.charAt( i + 1)];
            buffer[index++]= (byte)(((c0 << 2) | (c1 >> 4)) & mask);
            if(index >= buffer.length){
                return buffer;
            }
            int c2 = toInt[s.charAt( i + 2)];
            buffer[index++]= (byte)(((c1 << 4) | (c2 >> 2)) & mask);
            if(index >= buffer.length){
                return buffer;
            }
            int c3 = toInt[s.charAt( i + 3 )];
            buffer[index++]= (byte)(((c2 << 6) | c3) & mask);
        }
        return buffer;
    } 

    public static void main(String[] args)
    {
        Base64 base64 = new Base64();
        System.out.print("private int[] toInt = {");
        for(int i=0;i<base64.toInt.length;i++)
        {
             System.out.print(base64.toInt[i]);
             if(i<base64.toInt.length-1)
             System.out.print(",");
        }
        System.out.println("};\n");

        System.out.println("\n\nTest der Wandlung UTF8-codierter Strings:");

        String ttt="Daß es unerläßlich ist, auch UTF8-kodierte String wandeln zu können, sieht man hier.";

        System.out.println("Ausgangstext:\n");
        System.out.println(ttt);
        String tttw = base64.encodeUTF8(ttt);
        System.out.println("gewandelt:");
        System.out.println(tttw);
        System.out.println("wiederhergestellt:");
        String ttth = base64.decodeUTF8(tttw);
        System.out.println(ttth);
    }
}
