package kiba300.hilfsklassen;


import java.util.ArrayList;
import java.io.File;
import java.util.Vector;

 /**
 *  <h4>Parser.java</h4>
 *
 *  Copyright (C) 2013 Guido Kramann<br/>
 *  kramann@fh-brandenburg.de<br/>
 *  http://www.kramann.info<br/>
 *  <br/>
 *  <p>  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  </p>
 *  <p>
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  </p>
 *  <p>
 *  You should have received a copy of the GNU General Public License
 *  along with this program (gpl.html).  If not, see http://www.gnu.org/licenses/.
 *  </p>
 */
public class Parser
{
    /**
    Ersetzt Umlaute in Latex-spezifischer Weise.
    */
    public String wandelLatexUmlaute(String text)
    {
        StringBuffer sp = new StringBuffer();
        for(int i=0;i<text.length();i++)
        {
            char c = text.charAt(i);
            if(c=='ä')
                sp.append("\\\"{a}");
            else if(c=='ö')
                sp.append("\\\"{o}");
            else if(c=='ü')
                sp.append("\\\"{u}");
            else if(c=='Ä')
                sp.append("\\\"{A}");
            else if(c=='Ö')
                sp.append("\\\"{O}");
            else if(c=='Ü')
                sp.append("\\\"{U}");
            else if(c=='ß')
                sp.append("\\{ss}");
            else
                sp.append(c);
        }
        return sp.toString();
    }


    /**
    Die Methode extrahiereMusterloesung() ist eine spezielle Methode für E-Test-Programmieraufgaben,
    wie testc, um aus den Aufgaben-Quelltexten mit Eingabefeld-, Scheincode- und unsichtbarem Code
    einen lauffähigen Quelltext zu extrahieren, der der Musterlösung entspricht und dessen Ausgabe
    mit der bei Eintragungen durch die Studierenden sich ergebenden Ausgaben verglichen wird.<br/
    § Einfassung von Eingabefeldern mit angeklebter Punktzahl am schließenden Symbol!<br/>
    $ Einfassung von Scheincode<br/>
    @ Einfassung von unsichtbarem Code<br/>
    */
    public String extrahiereMusterloesung(String text)
    {
        String ergebnis = entferneScheincode(text);
        ergebnis = aktiviereUnsichtbarenCode(ergebnis);
        ergebnis = aktiviereMustercode(ergebnis);

        return ergebnis;
    }

    /**In $ eingefasster Code wird durch Leerzeichen ersetzt*/
    public String entferneScheincode(String text)
    {
        StringBuffer sb = new StringBuffer();

        if(text==null) return null;
        if(text.length()==0) return text;
        
        boolean scheincode=false;

        for(int i=0;i<text.length();i++)
        {
            char c = text.charAt(i);
            if(c=='$')
            {
                scheincode=!scheincode;
                c=' ';
            }
            else if(scheincode==true)
            {
                c=' ';
            }
            sb.append(c);
        }
 
        return sb.toString();    
    }

    /**In @ eingefasster Code wird durch Leerzeichen ersetzt*/
    public String entferneUnsichtbarenCode(String text)
    {
        StringBuffer sb = new StringBuffer();

        if(text==null) return null;
        if(text.length()==0) return text;
        
        boolean scheincode=false;

        for(int i=0;i<text.length();i++)
        {
            char c = text.charAt(i);
            if(c=='@')
            {
                scheincode=!scheincode;
                c=' ';
            }
            else if(scheincode==true)
            {
                c=' ';
            }
            sb.append(c);
        }
 
        return sb.toString();    
    }

    /**In @-Symbole durch Leerzeichen ersetzen.*/
    public String aktiviereUnsichtbarenCode(String text)
    {
        StringBuffer sb = new StringBuffer();

        if(text==null) return null;
        if(text.length()==0) return text;
        
        for(int i=0;i<text.length();i++)
        {
            char c = text.charAt(i);
            if(c=='@')
            {
                c=' ';
            }
            sb.append(c);
        }
 
        return sb.toString();    
    }

    /**In $-Symbole durch Leerzeichen ersetzen.*/
    public String aktiviereScheincode(String text)
    {
        StringBuffer sb = new StringBuffer();

        if(text==null) return null;
        if(text.length()==0) return text;
        
        for(int i=0;i<text.length();i++)
        {
            char c = text.charAt(i);
            if(c=='$')
            {
                c=' ';
            }
            sb.append(c);
        }
 
        return sb.toString();    
    }


    /**In §-Symbole und ggf. nachfolgende Punktezahlen durch Leerzeichen ersetzen.*/
    public String aktiviereMustercode(String text)
    {
        StringBuffer sb = new StringBuffer();
        boolean eingabefeld = false;
        int zahlenfeldlaenge = 0;
        if(text==null) return null;
        if(text.length()==0) return text;
        
        for(int i=0;i<text.length();i++)
        {
            char c = text.charAt(i);
            if(c=='§')
            {
                if(eingabefeld==true)
                {
                    zahlenfeldlaenge=0;
                    for(int k=i+1;k<text.length();k++)
                    {
                        if(istZahlenfeld(text.charAt(k)))
                            zahlenfeldlaenge++;
                        else
                            break;
                    }
                }

                eingabefeld=!eingabefeld;
                c=' ';
            }
            else if(zahlenfeldlaenge>0)
            {
                c=' ';
                zahlenfeldlaenge--;
            }
            sb.append(c);
        }
 
        return sb.toString();    
    }

    /**Trägt eine studentische Lösung an der Stelle index ein.<br/>
       Voraussetzung ist, dass die Zeilen- und Spaltenzahlen zwischen Quelle und Ziel übereinstimmen.<br/>
       index: Indexnummer des Eingabefeldes innerhalb des zugehörigen Quelltextes<br/>
       Die klammernden Zeichen §, $, @ werden in dieser Methode nicht entfernt. - Dies muss danach mit Hilfe der anderen Methoden geschehen.
    */
    public String trageStudentischeLoesungEin(String text, char[][] studlsg, int index)
    {
        StringBuffer sb = new StringBuffer();
        boolean eingabefeld = false;
        if(text==null) return null;
        if(text.length()==0) return text;

        int indexzaehler=0;

        //Quelltext in ein zweidimensionales char-Array umwandeln:        
        char[][] arr = stringToCharArray(text);

        int feldx=0;
        int feldy=0;

        boolean fertig = false;

        for(int i=0;i<arr.length;i++)
        {
            for(int k=0;k<arr[i].length;k++)
            {
                char c = arr[i][k];

                if(c=='§')
                {
                   eingabefeld=!eingabefeld; 

                   if(eingabefeld==true)
                   {
                       if(indexzaehler==index)
                       {
                            //richtige Stelle merken:
                            feldx=k+1;
                            feldy=i;
                            fertig = true;
                       }

                       indexzaehler++;
                   }
                }
                if(fertig) break;
            }
            if(fertig) break;
        }

        //Studentisches Feld an der richtigen Stelle einfügen:
        for(int i=0;i<studlsg.length;i++)
        {
            //Wenn Vorlage schmaler als Eintrag, dann Vorlage verbreitern:
            if(arr[feldy+i].length<studlsg[i].length+feldx)
            {
                int zusatz = studlsg[i].length+feldx - arr[feldy+i].length;
                char[] xx = new char[arr[feldy+i].length+zusatz];
                for(int uu=0;uu<arr[feldy+i].length;uu++)
                    xx[uu]=arr[feldy+i][uu];
                for(int uu=0;uu<zusatz;uu++)
                    xx[uu+arr[feldy+i].length]=' ';
                arr[feldy+i]=xx;                
            }

            for(int k=0;k<studlsg[i].length;k++)
            {
                 arr[feldy+i][feldx+k]=studlsg[i][k];
            }
        }

        //Array zurückwandeln:
        for(int i=0;i<arr.length;i++)
        {
            sb.append(new String(arr[i]));
            sb.append("\n");
        }

        return sb.toString();        
    }

    /**
    Hilfsmethode, um einen String in ein zweidimensionales Array aus Character-Zeichen umzuwandeln
    */       
    public char[][] stringToCharArray(String text)
    {
        String[] zeil = zerlegeZeilenNeu(text);

        if(zeil==null || zeil.length==0)
            return new char[][] {{}};

        char[][] erg = new char[zeil.length][];
        for(int i=0;i<zeil.length;i++)
            erg[i] = zeil[i].toCharArray();

        return erg;
    }

    /**
    Erwartet einen String der Länge 9, wobei die ersten und die letzten 4 Stellen eine Zahl repräsentieren müssen.<br/>
    Dazwischen steht ein /, das Format sollfolgendermassen aussehen: xx.x/xx.x, also z.B. 05.5/10.0.
    */
    public String formatierePunkteverteilung(String text)
    {
        if(text==null || text.length()!=9) return "00.0/00.0";

        char[] tt = text.toCharArray();

        //Führende Nullen entfernen:
        for(int i=0;i<tt.length-1;i++)
            if(tt[i]=='0' && tt[i+1]=='0')
                tt[i]=' ';

        double x=0.0;
        double y=0.0;

        char[] u1=new char[] {tt[0],tt[1],tt[2],tt[3]};
        char[] u2=new char[] {tt[5],tt[6],tt[7],tt[8]};

        try
        {
            x = Double.parseDouble(new String(u1));
            y = Double.parseDouble(new String(u2));

        }
        catch(Exception e)
        {
            x=0.0;
            y=0.0;
        }
        if(x<0.0) x=0.0;
        if(y<0.0) y=0.0;
        if(x>99.5) x=99.5;
        if(y>99.5) y=99.5;

        x*=10.0;
        y*=10.0;

        int ix = (int)x;
        int iy = (int)y;

        ix/=5; //nur Schritte von 0.5 Punkten erlauben.
        ix*=5;

        iy/=5;
        iy*=5;

        int x1 = ix%10;
        int x2 = (ix/10)%10;
        int x3 = (ix/100)%10;

        int y1 = iy%10;
        int y2 = (iy/10)%10;
        int y3 = (iy/100)%10;

        return ""+x3+""+x2+"."+x1+"/"+y3+""+y2+"."+y1;        
    }

    /**
    Zweite Variante - trägt das Ergebnis direkt in ein mit übergebenes char-Array ein.<br/>
    Erwartet einen String der Länge 9, wobei die ersten und die letzten 4 Stellen eine Zahl repräsentieren müssen.<br/>
    Dazwischen steht ein /, das Format sollfolgendermassen aussehen: xx.x/xx.x, also z.B. 05.5/10.0.
    */
    public void formatierePunkteverteilung(double x, double y,char[][] ein)
    {
        if(x<0.0) x=0.0;
        if(y<0.0) y=0.0;
        if(x>99.5) x=99.5;
        if(y>99.5) y=99.5;

        x*=10.0;
        y*=10.0;

        int ix = (int)x;
        int iy = (int)y;

        ix/=5; //nur Schritte von 0.5 Punkten erlauben.
        ix*=5;

        iy/=5;
        iy*=5;

        int x1 = ix%10;
        int x2 = (ix/10)%10;
        int x3 = (ix/100)%10;

        int y1 = iy%10;
        int y2 = (iy/10)%10;
        int y3 = (iy/100)%10;

        ein[0][0]=(""+x3).charAt(0);
        ein[0][1]=(""+x2).charAt(0);
        ein[0][2]=(".").charAt(0);
        ein[0][3]=(""+x1).charAt(0);

        ein[0][4]=("/").charAt(0);

        ein[0][5]=(""+y3).charAt(0);
        ein[0][6]=(""+y2).charAt(0);
        ein[0][7]=(".").charAt(0);
        ein[0][8]=(""+y1).charAt(0);
    }

    public int rundeNullFuenfMalZehn(double x)
    {
        int y = (int)(x*10.0);
        y/=5;
        y*=5;

        return y;
    }

    /**
    liefert -1.0 zurück bei Fehlern.
    */
    public double parseZahl(char[] itext)
    {
        if(itext==null || itext.length==0)
            return -1.0;

        char[] text = new char[itext.length];
        for(int i=0;i<text.length;i++)
            text[i]=itext[i];

        //Führende Nullen löschen:
        for(int i=0;i<text.length;i++)
        {
            if( (i==0 || text[i-1]==' ') && text[i]=='0' )
                text[i]=' ';
        }
        try
        {
            double wert = Double.parseDouble(new String(text));
            return wert;
        }
        catch(Exception e)
        {
            return -1.0;
        }
    }

    public double extrahierePunktzahl(String text)
    {
        if(text==null || text.length()!=9) return 0.0;

        char[] tt = text.toCharArray();

        //Führende Nullen entfernen:
        for(int i=0;i<tt.length-1;i++)
            if(tt[i]=='0' && tt[i+1]=='0')
                tt[i]=' ';

        double x=0.0;
        double y=0.0;

        char[] u1=new char[] {tt[0],tt[1],tt[2],tt[3]};
        char[] u2=new char[] {tt[5],tt[6],tt[7],tt[8]};

        try
        {
            x = Double.parseDouble(new String(u1));
            y = Double.parseDouble(new String(u2));

        }
        catch(Exception e)
        {
            x=0.0;
            y=0.0;
        }
        if(x<0.0) x=0.0;
        if(y<0.0) y=0.0;
        if(x>99.5) x=99.5;
        if(y>99.5) y=99.5;

        x*=10.0;
        y*=10.0;

        int ix = (int)x;
        int iy = (int)y;

        ix/=5; //nur Schritte von 0.5 Punkten erlauben.
        ix*=5;

        iy/=5;
        iy*=5;

        return 0.1*(double)iy;        
    }

    public char[] formatierePunktezahl(double punkte)
    {
        char[] pp = new char[] {'0','0','.','0'};

        int ppp = (int)(punkte*10.0);
        ppp/=5;
        ppp*=5;

        pp[0] = (""+(ppp/100)).charAt(0);
        ppp%=100;
        pp[1] = (""+(ppp/10)).charAt(0);
        ppp%=10;
        pp[3] = (""+ppp).charAt(0);

        return pp;
    }

    /**
    liefert -1.0 bei Fehlern zurückt.
    */
    public double extrahiereErreichtePunktzahl(String text)
    {
        if(text==null || text.length()!=9) return -1.0;

        char[] tt = text.toCharArray();

        //Führende Nullen entfernen:
        for(int i=0;i<tt.length-1;i++)
            if(tt[i]=='0' && tt[i+1]=='0')
                tt[i]=' ';

        double x=0.0;
        double y=0.0;

        char[] u1=new char[] {tt[0],tt[1],tt[2],tt[3]};
        char[] u2=new char[] {tt[5],tt[6],tt[7],tt[8]};

        try
        {
            x = Double.parseDouble(new String(u1));
            y = Double.parseDouble(new String(u2));

        }
        catch(Exception e)
        {
            x=0.0;
            y=0.0;
            return -1.0;
        }
        if(x<0.0) x=0.0;
        if(y<0.0) y=0.0;
        if(x>99.5) x=99.5;
        if(y>99.5) y=99.5;

        x*=10.0;
        y*=10.0;

        int ix = (int)x;
        int iy = (int)y;

        ix/=5; //nur Schritte von 0.5 Punkten erlauben.
        ix*=5;

        iy/=5;
        iy*=5;

//        return 0.1*(double)iy;        
        return 0.1*(double)ix;        
    }

    /**
    liefert -1.0 bei Fehlern zurückt.
    */
    public double extrahiereMoeglichePunktzahl(String text)
    {
        if(text==null || text.length()!=9) return -1.0;

        char[] tt = text.toCharArray();

        //Führende Nullen entfernen:
        for(int i=0;i<tt.length-1;i++)
            if(tt[i]=='0' && tt[i+1]=='0')
                tt[i]=' ';

        double x=0.0;
        double y=0.0;

        char[] u1=new char[] {tt[0],tt[1],tt[2],tt[3]};
        char[] u2=new char[] {tt[5],tt[6],tt[7],tt[8]};

        try
        {
            x = Double.parseDouble(new String(u1));
            y = Double.parseDouble(new String(u2));

        }
        catch(Exception e)
        {
            x=0.0;
            y=0.0;
            return -1.0;
        }
        if(x<0.0) x=0.0;
        if(y<0.0) y=0.0;
        if(x>99.5) x=99.5;
        if(y>99.5) y=99.5;

        x*=10.0;
        y*=10.0;

        int ix = (int)x;
        int iy = (int)y;

        ix/=5; //nur Schritte von 0.5 Punkten erlauben.
        ix*=5;

        iy/=5;
        iy*=5;

        return 0.1*(double)iy;        
//        return 0.1*(double)ix;        
    }

    /** x=[0..9] */
    public char zifferToZeichen(int x)
    {
        if(x<0 || x>9) return ' ';

        if(x==0) return '0';
        if(x==1) return '1';
        if(x==2) return '2';
        if(x==3) return '3';
        if(x==4) return '4';
        if(x==5) return '5';
        if(x==6) return '6';
        if(x==7) return '7';
        if(x==8) return '8';
        if(x==9) return '9';

        return ' ';
    }

    public boolean istZahlenfeld(char c)
    {
        if(c=='0') return true;
        if(c=='1') return true;
        if(c=='2') return true;
        if(c=='3') return true;
        if(c=='4') return true;
        if(c=='5') return true;
        if(c=='6') return true;
        if(c=='7') return true;
        if(c=='8') return true;
        if(c=='9') return true;
        if(c=='.') return true;
        return false;
    }

    /**Kapseln von Programmquelltexten: Anführungszeichen, Backslashes so darstellen, dass sie
    innerhalb von Anführungszeichen innerhalb von einem Java-Programm stehen können.*/
    public String kapselProgrammquelltextzeile(String text)
    {
            StringBuffer tt=new StringBuffer();
            if(text!=null && text.length()>0)
            {
                for(int k=0;k<text.length();k++)
                {
                    char c = text.charAt(k);
                    if(c=='\"')
                        tt.append('\\');
                    else if(c=='\'')
                        tt.append('\\');
                    else if(c=='\\')
                        tt.append('\\');
                    tt.append(c);
                }
            }
            return tt.toString();
    }

    /**Um Überschriften in der Navigation sicher zu erzeugen, dürfen die Überschriften nicht
    zu lang sein. Mit nachfolgender Funtion werden sie auf die richtige Breite gebracht durch Abscneiden.*/
    public String kurzeUeberschrift(String ueberschrift, int zeichenbreite, int maximalebreite_in_pixeln)
    {
        ArrayList<String> woert = extrahiereWoerter(ueberschrift);        
        String ueber = "";
        String blank = "";
        for(int i=0;i<woert.size();i++)
        {
            if(ueber.length()*zeichenbreite+woert.get(i).length()*zeichenbreite+blank.length()*zeichenbreite<=maximalebreite_in_pixeln)
                ueber+=blank+woert.get(i);
            else if(ueber.length()*zeichenbreite+3*zeichenbreite<=maximalebreite_in_pixeln)
                ueber+="...";
            blank=" ";
        }
        return ueber;
    }

    /**Prüfen, ob ein String ein bestimmtes Zeichen enthält*/
    public boolean enthaeltZeichen(String text,char zeichen)
    {
        if(text==null) return false;
        if(text.length()==0) return false;

        for(int i=0;i<text.length();i++)
            if(text.charAt(i)==zeichen)
                return true;

        return false;
    }

    /**Schneidet den Dateinamen von einem Pfad ab und gibt ihn zurück
       ohne vorangestellten Slash.
    */
    public String getStueckHinterLetztemSlashBeiPfad(String pfad)
    {
        if(pfad==null) return null;
        pfad = pfad.trim();
        if(enthaeltZeichen(pfad,'/')==false) return pfad;
        if(pfad.charAt(pfad.length()-1)=='/') return "";

        for(int i=pfad.length()-1;i>=0;i--)
        {
            if(pfad.charAt(i)=='/')
            {
                return pfad.substring(i+1,pfad.length());
            }
        }                  
        return "";
    }

    /**Schneidet Dateinamen von Pfad ab und liefert den bloßen Pfad
    ohne Slash am Ende*/
    public String getStueckVorLetztemSlashBeiPfad(String pfad)
    {
        if(pfad==null) return null;
        pfad = pfad.trim();
        if(enthaeltZeichen(pfad,'/')==false) return pfad;
        if(pfad.charAt(pfad.length()-1)=='/') return "";

        for(int i=pfad.length()-1;i>=0;i--)
        {
            if(pfad.charAt(i)=='/')
            {
                return pfad.substring(0,i);
            }
        }                  
        return pfad;
    }

    //NEUER BEREICH:

    /**Vergleicht test mit einem 2d-Array mit Befehlsnamen.<br/>
    Der Prefix für alle Befehle wird extra übergeben und ist nicht
    mit den Befehlsnamen zusammen gespeichert.*/
    public boolean istBefehl(String test, String[][] befehl,String PREFIX)
    {
                for(int k=0;k<befehl.length;k++)
                {
                    for(int p=0;p<befehl[k].length;p++)
                    {
                        if(test.equals(PREFIX+befehl[k][p]))
                            return true;
                    }
                }
                return false;
    }


    /**Ersetzt in text überall alt gegen neu.*/
    public String ersetzeUeberallFormel(String alt, String neu,String text)
    {
        if(text.length()<alt.length())
            return text;

        StringBuffer sb = new StringBuffer();

        for(int i=0;i<text.length();i++)
        {
            if( text.length()-i>=alt.length() && text.substring(i,i+alt.length()).equals(alt) )
            {
                sb.append(neu);
                i+=alt.length()-1;
            }
            else
            {
                sb.append(text.charAt(i));
            }
        }

        return sb.toString();        
    }



    /**Ersetzt in text überall alt gegen neu.*/
    public String ersetzeUeberall(String text, String alt, String neu)
    {
        if(text.length()<alt.length())
            return text;

        StringBuffer sb = new StringBuffer();

        for(int i=0;i<text.length();i++)
        {
            if( text.length()-i>=alt.length() && text.substring(i,i+alt.length()).equals(alt) )
            {
                sb.append(neu);
                i+=alt.length()-1;
            }
            else
            {
                sb.append(text.charAt(i));
            }
        }

        return sb.toString();        
    }

    /**Ersetzt überall in einem String-Array text alt gegen neu.*/
    public String[] ersetzeUeberall(String[] text, String alt, String neu)
    {
        Vector<String> sb = new Vector<String>();

        for(int i=0;i<text.length;i++)
        {
            sb.add( ersetzeUeberall(text[i], alt, neu) );
        }

        return (String[])sb.toArray(new String[0]);
    }

    /**Anzeigen eines 2d-String-Arrays*/
    public void zeigeArray(String[][] arr)
    {
        for(int i=0;i<arr.length;i++)
        {
            System.out.println("Zeile Nr. "+i+":");
            for(int k=0;k<arr[i].length;k++)
            {
                System.out.println("    "+arr[i][k]);
            }
        }
    }

    /**Anzeigen eines 1d-String-Arrays*/
    public void zeigeZeilen(String[] lines)
    {
        System.out.println("ZEILEN:");
        for(int i=0;i<lines.length;i++)
        {
            System.out.print("Zeile Nr. "+i+":");
            System.out.println(lines[i]);
        }
    }

    /**
    Prüft, ob ein verwendetes Zeichen im Zeichenvorrat eines C-Programms
    liegt. Umlaute sind beispielsweise nicht erlaubt.<br/>
    ACHTUNG: Zeichen für Etest-Klammern weren zugelassen.
    */
    public  boolean istProgrammZeichen(char c)
    {
        int z = (int)c;
        if(  
              (z>='A' && z<='Z') || 
              (z>='a' && z<='z') || 
              (z>='0' && z<='9') || 
              c=='.' || c==',' || c==';' || c==':' ||
              c=='-' || c=='+' || c=='*' || c=='/' || c=='\\' ||
              c=='\"' || c=='\'' || c=='?' || c=='!' || c=='&' ||
              c=='$' || c=='~' || c=='[' || c==']' || c=='{' ||
              c=='}' || c=='(' || c==')' || c=='<' || c=='>' ||
              c=='|' || c=='%' || c=='^' || c=='_'  || c==' '  || c=='=' ||
              c=='@' || c=='§'
          )
            return true;
        else
            return false;
    }

    /**
    Prüft, ob ein verwendetes Zeichen im Zeichenvorrat eines C-Programms
    liegt. Umlaute sind beispielsweise nicht erlaubt.
    */
    public  char neutralisiereSonderzeichen(char c)
    {
        int z = (int)c;
        if(  
              istProgrammZeichen(c)
          )
            return c;
        else
            return ' ';
    }

    /**
    Prüft, ob Zeichen Teil eines Wortes sein kann*/
    public boolean istBuchstabe(char c)
    {
            if(
                  c=='a' || c=='b' || c=='c' || c=='d' || c=='e' || c=='f' || c=='g' ||
                  c=='h' || c=='i' || c=='j' || c=='k' || c=='l' || c=='m' || c=='n' ||
                  c=='o' || c=='p' || c=='q' || c=='r' || c=='s' || c=='t' || c=='u' ||
                  c=='v' || c=='w' || c=='x' || c=='y' || c=='z' || 

                  c=='A' || c=='B' || c=='C' || c=='D' || c=='E' || c=='F' || c=='G' ||
                  c=='H' || c=='I' || c=='J' || c=='K' || c=='L' || c=='M' || c=='N' ||
                  c=='O' || c=='P' || c=='Q' || c=='R' || c=='S' || c=='T' || c=='U' ||
                  c=='V' || c=='W' || c=='X' || c=='Y' || c=='Z' ||
                  c=='ä' || c=='ö' || c=='ü' || c=='Ä' || c=='Ö' || c=='Ü' || c=='ß'
             )
             return true;

        return false;
    }


    /**
    wandelt den Unicode-Namen einer Person in reines ASCII um.<br/>
    z.B. um Namen in Dateinamen umzuwandeln<br/>
    */
    public  String nameToASCII(String name)
    {
      try
      {
        StringBuffer sb = new StringBuffer();
        for(int i=0;i<name.length();i++)
        {
            char c = name.charAt(i);
            if(
                  c=='a' || c=='b' || c=='c' || c=='d' || c=='e' || c=='f' || c=='g' ||
                  c=='h' || c=='i' || c=='j' || c=='k' || c=='l' || c=='m' || c=='n' ||
                  c=='o' || c=='p' || c=='q' || c=='r' || c=='s' || c=='t' || c=='u' ||
                  c=='v' || c=='w' || c=='x' || c=='y' || c=='z' || 

                  c=='A' || c=='B' || c=='C' || c=='D' || c=='E' || c=='F' || c=='G' ||
                  c=='H' || c=='I' || c=='J' || c=='K' || c=='L' || c=='M' || c=='N' ||
                  c=='O' || c=='P' || c=='Q' || c=='R' || c=='S' || c=='T' || c=='U' ||
                  c=='V' || c=='W' || c=='X' || c=='Y' || c=='Z'

             )
            {
                sb.append(c);
            }
            else if(c==' ') sb.append('_');
            else if(c=='ä') sb.append("ae");
            else if(c=='Ä') sb.append("Ae");
            else if(c=='ö') sb.append("oe");
            else if(c=='Ö') sb.append("Oe");
            else if(c=='ü') sb.append("ue");
            else if(c=='Ü') sb.append("Ue");
            else if(c=='ß') sb.append("ss");
            else sb.append('X');             
        }

        if(sb!=null && sb.length()>0)
            return sb.toString();
        else
            return "?";
      }
      catch(Exception eee)
      {
        return "?";
      }
    }

    /** ..auch Ziffern zulassen.*/
    public  String nameToASCII2(String name)
    {
      if(name==null) return "";
      if(name.length()==0) return "";

      try
      {
        StringBuffer sb = new StringBuffer();
        for(int i=0;i<name.length();i++)
        {
            char c = name.charAt(i);
            if(
                  c=='a' || c=='b' || c=='c' || c=='d' || c=='e' || c=='f' || c=='g' ||
                  c=='h' || c=='i' || c=='j' || c=='k' || c=='l' || c=='m' || c=='n' ||
                  c=='o' || c=='p' || c=='q' || c=='r' || c=='s' || c=='t' || c=='u' ||
                  c=='v' || c=='w' || c=='x' || c=='y' || c=='z' || 

                  c=='A' || c=='B' || c=='C' || c=='D' || c=='E' || c=='F' || c=='G' ||
                  c=='H' || c=='I' || c=='J' || c=='K' || c=='L' || c=='M' || c=='N' ||
                  c=='O' || c=='P' || c=='Q' || c=='R' || c=='S' || c=='T' || c=='U' ||
                  c=='V' || c=='W' || c=='X' || c=='Y' || c=='Z' ||
                  c=='0' || c=='1' || c=='2' || c=='3' || c=='4' || c=='5' || 
                  c=='6' || c=='7' || c=='8' || c=='9'
             )
            {
                sb.append(c);
            }
            else if(c==' ') sb.append('_');
            else if(c=='ä') sb.append("ae");
            else if(c=='Ä') sb.append("Ae");
            else if(c=='ö') sb.append("oe");
            else if(c=='Ö') sb.append("Oe");
            else if(c=='ü') sb.append("ue");
            else if(c=='Ü') sb.append("Ue");
            else if(c=='ß') sb.append("ss");
            else sb.append('X');             
        }

        if(sb!=null && sb.length()>0)
            return sb.toString();
        else
            return "?";
      }
      catch(Exception eee)
      {
        return "?";
      }
    }

    /**
    Helfermethode:<br/>
    Zählt die Anzahl der Zeilenumbrueche
    */
    public  int zaehleZeilenumbrueche(String text)
    {
        int anzahl = 0;
        if(text==null)
            return 0;
        if(text.length()==0)
            return 0;
        for(int i=0;i<text.length();i++)
            if(text.charAt(i)=='\n')
                anzahl++;
        return anzahl;
    }

    /**
    Helfermethode:<br/>
    Bricht einen Text so um, dass die Wörter in einer Zeile nie über die vorgegebene Zeichenanzahl pro Zeile
    hinausragen.<br/>
    Voraussetzung ist, dass der übergebene Text bereits so vorliegt, dass zwischen zwei Wörtern genau
    ein Leerzeichen steht.<br/>
    An den notwendigen Stellen wird in dieser Methode einfach das Leerzeichen durch einen 
    Zeilenumbruch ersetzt.
    */
    public  String brecheTextUm(String intext,int maximale_zeichenanzahl_pro_zeile)
    {
        if(intext==null)
            return null;
        if(intext.length()==0)
            return intext;

        int letzter_zeilenumbruch = -1;
        StringBuffer text = new StringBuffer();
        for(int i=0;i<intext.length();i++)
        {
            char c = intext.charAt(i);
            if(intext.charAt(i)==' ')
            {
                int k=i+1;
                while(k<intext.length() && intext.charAt(k)!=' ')
                   k++;
                if(k-letzter_zeilenumbruch >= maximale_zeichenanzahl_pro_zeile)
                {
                   c='\n';
                   letzter_zeilenumbruch=i;
                }
            }
            text.append(c);
        }
        return (text.toString()).trim();
    }

    /**neuer besser, kann auch Whitespaces erkennen und zersägt auch Wörter, die länger als eine Zeile sind.<br/>
    ACHTUNG: Fängt bei der dritten Zeile an, da die ersten beiden die Kennung, ob es sich um einen Befehl handelt und den Befehlsnamen enthalten.
    */
    public  ArrayList<String> brecheTextUm(String[] inintext,int maximale_zeichenanzahl_pro_zeile)
    {
        if(inintext==null)
            return null;
        if(inintext.length<3)//erst der dritte Eintrag ist Text!
            return null;

        StringBuffer sb = new StringBuffer();
        
        //for(int i=0;i<inintext.length;i++) //b p Fließtext
//Erst ab dritten Eintrag beginnen:
        for(int i=2;i<inintext.length;i++) 
            sb.append(inintext[i]+" ");

        ArrayList<String> woert = extrahiereWoerter(sb.toString());

        if(woert==null)
            return null;
        if(woert.size()==0)
            return null;


        //woerter, die länger, als die maximale Zeichenanzahl sind zerschneiden:
        ArrayList<String> woerter = new ArrayList<String>();
        for(int i=0;i<woert.size();i++)
        {
            if(woert.get(i).length()>=maximale_zeichenanzahl_pro_zeile)
            {
                String w = woert.get(i);
                while(w.length()>maximale_zeichenanzahl_pro_zeile)
                {
                    String vorn = w.substring(0,maximale_zeichenanzahl_pro_zeile);
                    woerter.add(vorn);
                    w = w.substring(vorn.length(),w.length());
                }
                if(w.length()>0)
                    woerter.add(w);
            }
            else
            {
                woerter.add(woert.get(i));
            }
        }

        if(woerter==null)
            return null;
        if(woerter.size()==0)
            return null;

        ArrayList<String> zeilen  = new ArrayList<String>();

        int zeichenanzahl=0;
        int start=0;
        int woerter_verarbeitet=0;
//System.out.println("woerter.size()="+woerter.size());
        for(int i=0;i<woerter.size();i++)
        {
            zeichenanzahl+=woerter.get(i).length();
            
            if(zeichenanzahl>maximale_zeichenanzahl_pro_zeile)
            {
                StringBuffer sb3 = new StringBuffer();
                for(int k=start;k<i;k++)
                {
                    sb3.append(woerter.get(k));
//System.out.println("WORT ANHAENGEN:"+woerter.get(k));
                    if(k<i-1)
                        sb3.append(" ");
                    woerter_verarbeitet++;
                }
                zeilen.add(sb3.toString());
                zeichenanzahl=woerter.get(i).length();
                start=i;
            }
            zeichenanzahl++; //leerzeichen zwischen den Wörtern   
        }
        StringBuffer sb2 = new StringBuffer();
        for(int i=woerter_verarbeitet;i<woerter.size();i++)
        {
            sb2.append(woerter.get(i));
            if(i<woerter.size()-1)
               sb2.append(" ");
        }
        zeilen.add(sb2.toString());

        return zeilen;
    }


    /**
    Überall alt gegen neu austauschen.
    */
    public  String ersetze(String text, String alt,String neu)
    {
        StringBuffer mSB = new StringBuffer();
        
        for(int i=0;i<text.length();i++)
        {
            if(i<text.length()-alt.length()+1 && alt.equals(text.substring(i,i+alt.length())))
            {
                mSB.append(neu);
                i+=alt.length()-1;
            }
            else
            {
                mSB.append(text.charAt(i));
            }            
        }
        
        return mSB.toString();
    }

    /**String in Zeilen zerlegen und leere Zeilen löschen.*/
    public  String[] zerlegeZeilen(String text)
    {
        String[] zeilen = text.split("\n");
        int anz=0;
        for(int i=0;i<zeilen.length;i++)
        {
            zeilen[i]=zeilen[i].trim();
            if(zeilen[i].length()>0)
                anz++;
        }
        String[] ergebnis = new String[anz];
        anz=0;
        for(int i=0;i<zeilen.length;i++)
        {
            if(zeilen[i].length()>0)
            {
                ergebnis[anz]=zeilen[i];
                anz++;
            }
        }
        return ergebnis;
    }

    public String[] zerlegeZeilenNeu(String text)
    {
        int start=0;

        if(text==null || text.length()==0)
            return new String[] {};

        ArrayList<String> arr = new ArrayList<String>();

        for(int i=0;i<text.length();i++)
        {
            if(text.charAt(i)=='\n' || i==text.length()-1)
            {
                if(i==text.length()-1 && text.charAt(i)!='\n')
                    arr.add(text.substring(start,i+1));
                else
                    arr.add(text.substring(start,i));
                start=i+1;
            }
        }

        return arr.toArray(new String[]{});
    }

    /**String in Zeilen zerlegen und leere Zeilen NICHT löschen.*/
    public  String[] zerlegeZeilenExakt(char[] text)
    {
        int anz = 1;
        for(int i=0;i<text.length;i++)
            if(text[i]=='\n')
                anz++;

//System.out.println("gezaehlte Zeilenumbr. in zerlegeZeilenExakt(char[] text):"+anz);

        String[] erg = new String[anz];
        anz=0;
        for(int i=0;i<text.length;i++)
        {
            if(text[i]=='\n' || i==text.length-1)
            {
                int k=i;
                while(k-1>=0 && text[k-1]!='\n')
                    k--;
                if(k<i)
                {
                    if(i==text.length-1 && text[i]!='\n')
                       erg[anz] = new String(text,k,i-k+1);
                    else
                       erg[anz] = new String(text,k,i-k);
                    anz++;
                }
                else
                {
                    erg[anz] = new String(" ");
                    anz++;
                }
            }
        }


//System.out.println("hinterher:"+anz);

        return erg;
    }

    /**zerlegt einen Text, wo eine bestimmte Trenn-Zeichenfolge auftritt.*/
    public  ArrayList<String> zerlegeBeiTrenner(String text,String trenner)
    {
        if(text==null || text.length()==0 || trenner==null
       || trenner.length()==0 || trenner.length()>=text.length())
            return null;

        int index = 0;
        StringBuffer sb = null;

        ArrayList<String> merker = new ArrayList<String>();

        while(index<text.length())
        {
            if(sb==null)
                sb = new StringBuffer();

            if(index<text.length()-trenner.length()+1
            && text.substring(index,index+trenner.length()).equals(trenner))
            {
                index+=trenner.length()-1;
                merker.add(sb.toString());     
                sb = new StringBuffer();
//System.out.println("AAA");

                //So wäre es ganz korrekt:
                //Falls am Ende, oder nachfolgende Trenner vorhanden,
                //weitere Elemente anhängen:
            }
            else if(index>=text.length()-1)
            {
//System.out.println("BBB");
                sb.append(text.charAt(index));
                merker.add(sb.toString());     
                break;                
            }
            else
            {
                sb.append(text.charAt(index));
            }
            index++;
        }

        return merker;
    }

   /**Zerlegt einen String in einzelne Woerter*/
   public  ArrayList<String> extrahiereWoerter(String text)
   {
       int i=0;
       ArrayList<String> ergebnis = new ArrayList<String>();
       while(i<text.length())
       {
           char c = text.charAt(i);
           while(Character.isWhitespace(c))
           {
               i++;
               if(i>=text.length())
                   break;
               c = text.charAt(i);
           }
           StringBuffer buff = new StringBuffer(); 
           while(!Character.isWhitespace(c))
           {
               buff.append(c);
               i++;
               if(i>=text.length())
                   break;
               c = text.charAt(i);
           }
           if(buff.length()>0)
           {
               String b = buff.toString();
               b=b.trim();
               if(!b.equals(";") && !b.equals(",") && !b.equals(":") && !b.equals("|"))
                   ergebnis.add(buff.toString());
           }
       }
       return ergebnis;
   }

   /**True, wenn übergebenes Zeichen eine Ziffer oder ein Dezimalpunkt ist.*/
   public  boolean isZifferOderPunkt(char c)
   {
       if(c=='0') return true;
       if(c=='1') return true;
       if(c=='2') return true;
       if(c=='3') return true;
       if(c=='4') return true;
       if(c=='5') return true;
       if(c=='6') return true;
       if(c=='7') return true;
       if(c=='8') return true;
       if(c=='9') return true;
       if(c=='.') return true;
       return false;
   }
      /**True, wenn übergebenes Zeichen eine Ziffer, Vorzeichen oder ein Dezimalpunkt ist.*/
   public  boolean isZifferOderPunktOderVorzeichen(char c)
   {
       if(c=='0') return true;
       if(c=='1') return true;
       if(c=='2') return true;
       if(c=='3') return true;
       if(c=='4') return true;
       if(c=='5') return true;
       if(c=='6') return true;
       if(c=='7') return true;
       if(c=='8') return true;
       if(c=='9') return true;
       if(c=='.') return true;
       if(c=='+') return true;
       if(c=='-') return true;
       return false;
   }
   
   /** Extrahiert aus einem Text alle auftretenden IP-Adressen.*/
   public  ArrayList<String> extrahiereErlaubteIpAdressen(String text)
   {
       int i=0;
       ArrayList<String> ergebnis = new ArrayList<String>();
       while(text!=null && text.length()>0 && i<text.length())
       {
           char c = text.charAt(i);
           while(!isZifferOderPunkt(c))
           {
               i++;
               if(i>=text.length())
                   break;
               c = text.charAt(i);
           }
           StringBuffer buff = new StringBuffer(); 
           while(isZifferOderPunkt(c))
           {
               buff.append(c);
               i++;
               if(i>=text.length())
                   break;
               c = text.charAt(i);
           }
           if(buff.length()>0)
           {
               ergebnis.add(buff.toString());
           }
       }
       return ergebnis;
   }

   public static void main(String[] args)
   {
       System.out.println("Testen der Klasse kramann.hilfsklassen.Parser");

       Parser parser = new Parser();

       String pfad = "/mnt-system/Z_PROJEKTE_NEU/kramann/huhu.txt";

       System.out.println("Beispielpfad:"+pfad);
       System.out.println("Aufruf von getStueckHinterLetztemSlashBeiPfad(String pfad) liefert:");
       System.out.println(parser.getStueckHinterLetztemSlashBeiPfad(pfad));

       System.out.println("Aufruf von getStueckVorLetztemSlashBeiPfad(String pfad) liefert:");
       System.out.println(parser.getStueckVorLetztemSlashBeiPfad(pfad));

       System.out.println("Testen von Textumbruechen: ArrayList<String> liste = brecheTextUm(String[] inintext,int maximale_zeichenanzahl_pro_zeile)");
       System.out.println("Beispieltext:");
       String[] zeilen = {"Dies ist Zeile 1","012345678901234567890123456789","huhu1 huhu2 huhu3 huhu4 huhu5 huhu6 huhu7 huhu8 "};
       System.out.println(zeilen[0]);
       System.out.println(zeilen[1]);
       System.out.println(zeilen[2]);
       System.out.println("Gewünschte maximale Zeichenanzahl pro Zeile: 20:");
       System.out.println("01234567890123456789");
       ArrayList<String> liste = parser.brecheTextUm(zeilen,20);
       for(int i=0;i<liste.size();i++)
           System.out.println(liste.get(i));
   }
}
