package kiba300.linuxsystem;


import java.util.Random;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.Vector;
import java.util.ArrayList;

import kiba300.hilfsklassen.Parser;
import kiba300.avr.AVRIO;

 /**
 *  <h4>Linuxsystem.java</h4>
 *
 *  Copyright (C) 2013 Guido Kramann<br/>
 *  kramann@fh-brandenburg.de<br/>
 *  http://www.kramann.info<br/>
 *  <br/>
 *  <p>  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  </p>
 *  <p>
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  </p>
 *  <p>
 *  You should have received a copy of the GNU General Public License
 *  along with this program (gpl.html).  If not, see http://www.gnu.org/licenses/.
 *  </p>
 *  ACHTUNG: String aufruf(String) führt aus bisher nicht gefundenen Gründen manchmal zu Systemabstürzen.<br/>
 *           Deshalb ist der Befehl void syst(String) vorzuziehen.
 */
public final class Linuxsystem
{
    private Random zufall = new Random(System.currentTimeMillis());

    private Parser parser = new Parser();    

    /**
    Ruft einen Systembefehl auf und liefert dessen Rückgabe als String zurück.<br/><br/>
    Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath() <br/>
    liefert den absoluten Pfad des jar-Files, in dem sich Linuxsystem.class befindet.
    */
    public final native String aufrufen(String command);

    /**
    Einfacher Aufruf von "system(...)" von der eingebetteten C-Funktion aus.
    */
    public final native void syst(String command);

    public void systThread(String command)
    {
       SystThread st = new SystThread(command);
       (new Thread(st)).start();
    }

    /**Hilfsklasse zum Starten eines neuen Thzreads bei einem Aufruf.*/
    private final class SystThread implements Runnable
    {
         String bef;

         public SystThread(String command)
         {
             bef = command;
         }

         public void run()
         {
             syst(bef);
         }
    }


    public Linuxsystem()
    {
        AVRIO.init();
    }

    /**Nutzt public native void syst(String command), um 
       - das übergebene Kommando auszuführen,<br/>
       - dessen Ausgabe in eine Datei mit zufällig erzeugtem Namen zu speichern, <br/>
       - diese Datei auszulesen und den Inhalt zeilenweise zurückzuliefern, <br/>
       - die Datei wieder zu löschen.<br/>
       Hinweis: Es werden der Standard- und der Fehlerkanal in die Zieldatei geschrieben.
    */
    public final String[] call(String command)
    {
        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);

        long x = zufall.nextLong();
        
        pfad+="/.temporaer"+x;

        syst(command+" > "+pfad+" 2>&1");        

        String[] ergebnis = null;
        Vector<String> vektor;
        File datei = new File(pfad);
        if(datei.exists()==false)
            return null;

        try
        {
            vektor = new Vector<String>();

            FileReader meinLeser = new FileReader( new File(pfad) );
            BufferedReader pufferleser  = new BufferedReader(meinLeser);

            String zeile = pufferleser.readLine();
            while( zeile!=null )
            {
                vektor.add(zeile);
                zeile = pufferleser.readLine();
            }
            meinLeser.close();
            syst("rm -f "+pfad);
            ergebnis = (String[])vektor.toArray(new String[0]);
            return ergebnis;
        }
        catch(IOException e) 
        { 
            syst("rm -f "+pfad);
            return null;
        }
    }

    public final String callString(String command)
    {
        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
        
        pfad+="/.temporaer"+x;

        syst(command+" > "+pfad+" 2>&1");        

        StringBuffer ergebnis = new StringBuffer();
        File datei = new File(pfad);
        if(datei.exists()==false)
            return null;

        try
        {
            FileReader meinLeser = new FileReader( new File(pfad) );
            BufferedReader pufferleser  = new BufferedReader(meinLeser);

            int zeile = pufferleser.read();
            while( zeile!=-1 )
            {
                ergebnis.append((char)zeile);
                zeile = pufferleser.read();
            }
            meinLeser.close();
            syst("rm -f "+pfad);
            return ergebnis.toString();
        }
        catch(IOException e) 
        { 
            syst("rm -f "+pfad);
            return null;
        }        
    }

    public final byte[] callBytes(String command)
    {
        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
        
        pfad+="/.temporaer"+x;

        syst(command+" > "+pfad+" 2>&1");        

        try
        {
            File meinFile = new File(pfad);
            FileInputStream meinStream = new FileInputStream(meinFile);
            int anzahl = meinStream.available();
            byte[] daten = new byte[ anzahl ];            
            meinStream.read( daten );
            meinStream.close();
            syst("rm -f "+pfad);
            return daten;
        }
        catch(IOException e)
        {
            System.out.println("Fehler beim Laden:");
            System.out.println(e.getMessage());
            syst("rm -f "+pfad);
            return null;            
        }        
    }

//******************
//***** C **********
//******************

    /**
    Kompiliert den übergebenen Quelltext mit gcc und läßt das Programm laufen.<br/>
    Zurückgegeben werden zwei Strings:<br/>
    Das erste enthält die Rückgabe von dem Compiler und das zweite die vom Programm.
    */
    public final String[] gccNormal(String quelltext)
    {
        String[] erg = new String[2];

        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
     
        String name = "temporaer"+x;
   


        speichern(pfad+"/"+name+".c",quelltext);        

        erg[0] = callString("gcc -o "+pfad+"/"+name+" "+pfad+"/"+name+".c");
        syst("rm -f "+pfad+"/"+name+".c");
        erg[1] = callString(pfad+"/"+name);
        syst("rm -f "+pfad+"/"+name);



        return erg;
    }

    /**Läßt die Ausführung des erfolgreich kompilierten Programms in einem Thread laufen und
    wartet nach der übergebenen Millisekundenzahl nicht länger auf eine Antwort.<br/>
    Diese Variante kann dazu dienen Schäden zu vermeiden, die durch Endlosschleifen in Quelltexten 
    verursacht werden können.<br/>
    Es empfiehlt sich, in einem unsichtbaren Code eine sleep() Funktion in Schleifen einzubauen,
    damit bei Endlosschleifen ggf. der Prozessor nicht zu sehr durch den weiter laufenden Prozess
    belastet wird.*/
    public final String[] gccBegrenzt(String quelltext,long max_dauer_millis)
    {
        ThreadGccBegrenzt tgb = new ThreadGccBegrenzt(quelltext);
        try
        {
            Thread thread = new Thread(tgb);
            thread.start();
            long zeit=System.currentTimeMillis();
            while( System.currentTimeMillis()<zeit+max_dauer_millis && tgb.ergebnisT==null )
            {
                Thread.sleep(50);
            }

            if(tgb.ergebnisT!=null)
                return tgb.ergebnisT;
            else
                return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert.",null};
        }
        catch(Exception e)
        {
            return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert und es traten Fehler beim Threadding auf.",null};
        }
    }    

    /**Hilfsklasse für gccBegrenzt, über die das Threadding realisiert wird.*/
    private final class ThreadGccBegrenzt implements Runnable
    {
         public String[] ergebnisT = null;
         public String in_quelltext = null;

         public ThreadGccBegrenzt(String quelltext)
         {
             in_quelltext = quelltext;           
         }

         public void run()
         {
             ergebnisT=gccNormal(in_quelltext);
         }
    }

    /**gcc wird von kramann.bauteile.etest.testc aufgerufen, um eine C-Programmieraufgabe mit
    Eingabefeldern zu realisieren.<br/>
    Da es nicht selten vorkommt, dass Studierende Endlosschleifen programmieren, kann neben der
    normalen Kompilerfunktion gccNormal() auch eine alternative gccBegrenzt aufgerufen werden,
    die die Ausführdauer eines Programmaufrufs auf 20 Sekunden begrenzt.<br/>
    Die Auswahl erfordert ein neues Kompilieren des Projekts (s. Quelltext dieser Methode).<br/>
    */
    public final String[] gcc(String quelltext)
    {
        //Normales Kompilieren und Ausführen:
        //return gccNormal(quelltext);

        //Zeitliche Begrenzung des Ausführens um die übergebene Millisekundenzeit:
        return gccBegrenzt(quelltext,5000);
    }

//******************
//***** C++ ********
//******************

    public final String[] gppNormal(String quelltext)
    {
        String[] erg = new String[2];

        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
     
        String name = "temporaer"+x;
   


        speichern(pfad+"/"+name+".cpp",quelltext);        

        erg[0] = callString("g++ -o "+pfad+"/"+name+" "+pfad+"/"+name+".cpp");
//        erg[0] = callString("export PATH=\"/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games\";g++ -o "+pfad+"/"+name+" "+pfad+"/"+name+".cpp");
        syst("rm -f "+pfad+"/"+name+".cpp");
        erg[1] = callString(pfad+"/"+name);
        syst("rm -f "+pfad+"/"+name);



        return erg;
    }

    public final String[] gpp(String quelltext)
    {
        //Normales Kompilieren und Ausführen:
        //return gppNormal(quelltext);

        //Zeitliche Begrenzung des Ausführens um die übergebene Millisekundenzeit:
        return gppBegrenzt(quelltext,5000);
    }

    public final String[] gppBegrenzt(String quelltext,long max_dauer_millis)
    {
        ThreadGppBegrenzt tgb = new ThreadGppBegrenzt(quelltext);
        try
        {
            Thread thread = new Thread(tgb);
            thread.start();
            long zeit=System.currentTimeMillis();
            while( System.currentTimeMillis()<zeit+max_dauer_millis && tgb.ergebnisT==null )
            {
                Thread.sleep(50);
            }

            if(tgb.ergebnisT!=null)
                return tgb.ergebnisT;
            else
                return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert.",null};
        }
        catch(Exception e)
        {
            return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert und es traten Fehler beim Threadding auf.",null};
        }
    }    

    /**Hilfsklasse für gppBegrenzt, über die das Threadding realisiert wird.*/
    private final class ThreadGppBegrenzt implements Runnable
    {
         public String[] ergebnisT = null;
         public String in_quelltext = null;

         public ThreadGppBegrenzt(String quelltext)
         {
             in_quelltext = quelltext;           
         }

         public void run()
         {
             ergebnisT=gppNormal(in_quelltext);
         }
    }

//******************
//***** JAVA *******
//******************

    public final String[] javacNormal(String quelltext)
    {
        String[] erg = new String[2];

        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();

        if(x<0)
           x=-x;
     
        String name = "temporaer"+x;
   
        //Quelltext mit Klasse wrappen, die den Namen der Zufallsdatei trägt:
        String[] zzz = parser.zerlegeZeilenNeu(quelltext);

        //import-Dateien und package-Angaben übergehen:
        int u=0;
        while(zzz!=null && zzz.length>u && 
              (
                 ( zzz[u].trim().length()>"import".length() && zzz[u].trim().substring(0,"import".length()).equals("import") )
              || ( zzz[u].trim().length()>"package".length() && zzz[u].trim().substring(0,"package".length()).equals("package") )
              || ( zzz[u].trim().length()==0 )
              )
             )
        {
            u++;
        }

        ArrayList<String> www = parser.extrahiereWoerter(zzz[u]);
        String klassenname = null;
        if(www.size()>1 && www.get(0).equals("class")) klassenname=www.get(1);
        if(www.size()>2 && www.get(1).equals("class")) klassenname=www.get(2);
        if(www.size()>3 && www.get(2).equals("class")) klassenname=www.get(3);
        if(www.size()>4 && www.get(3).equals("class")) klassenname=www.get(4);

        //Zeile mit statischer main-Methode suchen und "static" entfernen:
        int uu=u;
        String mainzeile=null;
        while(uu<zzz.length)
        {
            ArrayList<String> mmm = parser.extrahiereWoerter(zzz[uu]);            
            if(mmm!=null && mmm.size()>=4 
                 && mmm.get(0).trim().equals("public")
                 && mmm.get(1).trim().equals("static")
                 && mmm.get(2).trim().equals("void")
                 && mmm.get(3).trim().substring(0,4).equals("main")
              )
            {
                mainzeile = "public void main(String[] args)";
                break;
            }
            uu++;
        }

        if(mainzeile==null)
        {
            System.out.println("Bei Java-Aufgaben muß die erste Klasse eine main-Methode enthalten.");
            System.out.println("Diese wurde nicht gefunden.");
            System.out.println("Ansonsnten dürfen die Klassen keine statischen Elemente enthalten!");
            System.out.println("Das Programm terminiert jetzt, um keine nicht funktionierenden Aufgaben zuzulassen.");
            System.exit(0);
        }

        System.out.println("Linuxsystem.javacNormal()");
        System.out.println("Kompilieren eines Java-Quelltextes.");
        System.out.println("Kompilieren eines Java-Quelltextes.");
        if(klassenname!=null)
        {
            System.out.println("Es wurde folgender Klassenname gefunden:"+klassenname);
        }
        else
        {
            System.out.println("Es konnte kein Klassenname gefunden werden.");
            System.out.println("ACHTUNG:");
            System.out.println("Beim Erstellen von Java-Aufgaben muß eine besondere Form eingehalten werden:");
            System.out.println("Es können mehrere Klassen hintereinander folgen.");
            System.out.println("Die erste Klasse muß eine main-Mathode enthalten.");
            System.out.println("Das Programm terminiert jetzt, um keine nicht funktionierenden Aufgaben zuzulassen.");
            System.exit(0);
        }

        StringBuffer sq = new StringBuffer();
        for(int ii=0;ii<u;ii++)
        {
            sq.append(zzz[ii]);
            sq.append("\n");
        }

        //... mit Wrapperklasse umhüllen:

        sq.append("public class "+name+"\n");
        sq.append("{\n");

        for(int ii=u;ii<zzz.length;ii++)
        {
            if(ii==uu)
                sq.append(mainzeile);
            else
                sq.append(zzz[ii]);

            sq.append("\n");
        }

        //Objekt der zuerst gefundenen Klasse bilden und von dieser die main-Methode aufrufen:

        sq.append("    public void testen()");
        sq.append("    {\n");
        sq.append("        "+klassenname+" mein"+klassenname+"= new "+klassenname+"();\n");
        sq.append("        mein"+klassenname+".main(new String[] {});\n");
        sq.append("    }\n");

        sq.append("    public static void main(String[] args)");
        sq.append("    {\n");
        sq.append("        "+name+" mein"+name+"= new "+name+"();\n");
        sq.append("        mein"+name+".testen();\n");
        sq.append("    }\n");

        sq.append("}\n");

        quelltext = sq.toString();
        
        System.out.println("Es wird nun das folgende Java-Programm kompiliert und gestartet:");
        System.out.println(quelltext);

System.out.println("SPEICHERN:"+pfad+"/"+name+".java");

        speichern(pfad+"/"+name+".java",quelltext);        

System.out.println("AUFRUFEN:"+"javac -d "+pfad+" "+pfad+"/"+name+".java");
//Pfad für class-File mit -d spezifizieren:
//        erg[0] = callString("javac "+pfad+"/"+name+".java");
//        erg[0] = callString("javac -d "+pfad+" "+pfad+"/"+name+".java");
        erg[0] = callString("javac "+pfad+"/"+name+".java -d "+pfad);
//        erg[0] = callString("cd "+pfad+";javac "+pfad+"/"+name+".java");
//        syst("rm -f "+pfad+"/"+name+".java");
System.out.println("AUFRUFEN2:"+"cd "+pfad+"; java "+name);
        erg[1] = callString("java -cp "+pfad+" "+name);
//        erg[1] = callString("java "+pfad+"/"+name);
//        erg[1] = callString("cd "+pfad+"; java "+name);
//        syst("rm -f "+pfad+"/"+name+".class");
//        syst("rm -f "+pfad+"/"+name+"*.class");



        return erg;
    }

    public final String[] javac(String quelltext)
    {
        //Normales Kompilieren und Ausführen:
        //return javacNormal(quelltext);

        //Zeitliche Begrenzung des Ausführens um die übergebene Millisekundenzeit:
        return javacBegrenzt(quelltext,5000);
    }

    public final String[] javacBegrenzt(String quelltext,long max_dauer_millis)
    {
        ThreadJavacBegrenzt tgb = new ThreadJavacBegrenzt(quelltext);
        try
        {
            Thread thread = new Thread(tgb);
            thread.start();
            long zeit=System.currentTimeMillis();
            while( System.currentTimeMillis()<zeit+max_dauer_millis && tgb.ergebnisT==null )
            {
                Thread.sleep(50);
            }

            if(tgb.ergebnisT!=null)
                return tgb.ergebnisT;
            else
                return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert.",null};
        }
        catch(Exception e)
        {
            return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert und es traten Fehler beim Threadding auf.",null};
        }
    }    

    /**Hilfsklasse für javacBegrenzt, über die das Threadding realisiert wird.*/
    private final class ThreadJavacBegrenzt implements Runnable
    {
         public String[] ergebnisT = null;
         public String in_quelltext = null;

         public ThreadJavacBegrenzt(String quelltext)
         {
             in_quelltext = quelltext;           
         }

         public void run()
         {
             ergebnisT=javacNormal(in_quelltext);
         }
    }

//******************
//***** SCILAB *****
//******************

    /**
    Abarbeiten von Scilab-Skripten, die als String übergeben werden.<br/>
    ACHTUNG: Es wird ein try-catch-Block ergänzt, um Fehler abzufangen und
    es wird ein Silent-Mode bei der Abarbeitung gewählt. Um Ergebnisse sehen zu können müssen diese
    mit disp() (Display/Anzeigen) dargestellt werden. Beispiel: Variable x enthält die relevanten Ergebnisse, also
    sollte das Skript am Ende disp(x); enthalten.
    */
    public final String[] scilabNormal(String quelltext)
    {
        String[] erg = new String[2];

        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
        if(x<0)
            x=-x;     

        String name = "temporaer"+x;
   
        //Quelltext ergänzen: try-Block und quit
        quelltext = "try\n"
                  + quelltext
                  + "\ncatch\n"
                  + "    disp(\'ERROR - Bei der Abarbeitung des Scilab-Scriptes trat ein Fehler auf.\');\n"
                  + "end\n"
                  + "quit;\n";

        speichern(pfad+"/"+name+".sce",quelltext);        

        //-1 Option: keine Meldungen, nur wenn explizit disp() aufgerufen wird.
//        erg[1] = callString("scilab  -nouserstartup -nb -nwni -nogui  -e \"exec(\'"+pfad+"/"+name+".sce\',-1)\"");
//21.10.2013, Version für Scilab 4:
        erg[1] = callString("scilab -ns -nb -nwni -nogui -e \"exec(\'"+pfad+"/"+name+".sce\',-1)\"");
        syst("rm -f "+pfad+"/"+name+".sce");

        if(erg[1]==null || erg[1].trim().length()==0 
                        || (erg[1].trim().length()>="ERROR".length() && erg[1].trim().substring(0,"ERROR".length()).equals("ERROR")))
        {
            if(erg[1].trim().length()>="ERROR".length() && erg[1].trim().substring(0,"ERROR".length()).equals("ERROR"))
                erg[0]=erg[1];
            else
                erg[0]="ERROR - Bei der Abarbeitung des Scilab-Scriptes trat ein Fehler auf.";
            erg[1]="";
        }
        else
        {
            erg[0]="";
        }

        return erg;
    }

    public final String[] scilab(String quelltext)
    {
        //Normales Kompilieren und Ausführen:
        //return scilabNormal(quelltext);

        //Zeitliche Begrenzung des Ausführens um die übergebene Millisekundenzeit:
        return scilabBegrenzt(quelltext,5000);
    }

    public final String[] scilabBegrenzt(String quelltext,long max_dauer_millis)
    {
        ThreadScilabBegrenzt tgb = new ThreadScilabBegrenzt(quelltext);
        try
        {
            Thread thread = new Thread(tgb);
            thread.start();
            long zeit=System.currentTimeMillis();
            while( System.currentTimeMillis()<zeit+max_dauer_millis && tgb.ergebnisT==null )
            {
                Thread.sleep(50);
            }

            if(tgb.ergebnisT!=null)
                return tgb.ergebnisT;
            else
                return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert.",null};
        }
        catch(Exception e)
        {
            return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert und es traten Fehler beim Threadding auf.",null};
        }
    }    


    /**Hilfsklasse für scilabBegrenzt, über die das Threadding realisiert wird.*/
    private final class ThreadScilabBegrenzt implements Runnable
    {
         public String[] ergebnisT = null;
         public String in_quelltext = null;

         public ThreadScilabBegrenzt(String quelltext)
         {
             in_quelltext = quelltext;           
         }

         public void run()
         {
             ergebnisT=scilabNormal(in_quelltext);
         }
    }

//*********************
//***** avrgcc ********
//*********************

    public final String[] avrgccNormal(String quelltext)
    {
        String[] erg = new String[2];

        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
     
        String name = "temporaer"+x;
   
        //Bitdarstellung einfach überall durch Dezimalzahlen ersetzen:
        //Also 0b00000000 durch 0, 0b00000001 durch 1 usw.:
        char[] bbb=new char[8];
        for(int qq=0;qq<256;qq++)
        {
            int ii=qq;
            for(int k=0;k<8;k++)
            {
                if(ii%2==1)
                    bbb[7-k]='1';
                else
                    bbb[7-k]='0';
                ii/=2;
            }

            String ersetze = "0b"+(new String(bbb));
            quelltext = parser.ersetze(quelltext,ersetze,""+qq);
        }

        //Quelltext vorverarbeiten:
        //Die Zeile #include<avr/io.h> wird ersetzt durch den Inhalt von kramann.avr.AVRIO:
        //Es wird davon ausgegangen, dass diese Include-Zeile die erste Zeile ist.
        String[] zeilen = parser.zerlegeZeilenNeu(quelltext);

        StringBuffer sb = new StringBuffer();

        sb.append("#include<stdio.h>");
        sb.append("\n");

        if(AVRIO.pdetemp==null)
            AVRIO.init();
        for(int ii=0;ii<AVRIO.pdetemp.length;ii++)
        {
            sb.append(AVRIO.pdetemp[ii]);
            sb.append("\n");
        }

        for(int ii=0;ii<zeilen.length;ii++)
        {
            if(zeilen[ii].trim().length()>="#include".length() 
            && zeilen[ii].trim().substring(0,"#include".length()).equals("#include"))
            { 
            }
            else 
            {
                sb.append(zeilen[ii]);
                sb.append("\n");
            }
        }

        quelltext = sb.toString();

        speichern(pfad+"/"+name+".c",quelltext);        

        erg[0] = callString("gcc -o "+pfad+"/"+name+" "+pfad+"/"+name+".c");
//        syst("rm -f "+pfad+"/"+name+".c");
        erg[1] = callString(pfad+"/"+name);
        syst("rm -f "+pfad+"/"+name);

        return erg;
    }

    public final String[] avrgcc(String quelltext)
    {
        //Normales Kompilieren und Ausführen:
        //return avrgccNormal(quelltext);

        //Zeitliche Begrenzung des Ausführens um die übergebene Millisekundenzeit:
        return avrgccBegrenzt(quelltext,5000);
    }

    public final String[] avrgccBegrenzt(String quelltext,long max_dauer_millis)
    {
        ThreadAvrgccBegrenzt tgb = new ThreadAvrgccBegrenzt(quelltext);
        try
        {
            Thread thread = new Thread(tgb);
            thread.start();
            long zeit=System.currentTimeMillis();
            while( System.currentTimeMillis()<zeit+max_dauer_millis && tgb.ergebnisT==null )
            {
                Thread.sleep(50);
            }

            if(tgb.ergebnisT!=null)
                return tgb.ergebnisT;
            else
                return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert.",null};
        }
        catch(Exception e)
        {
            return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert und es traten Fehler beim Threadding auf.",null};
        }
    }    

    /**Hilfsklasse für avrgccBegrenzt, über die das Threadding realisiert wird.*/
    private final class ThreadAvrgccBegrenzt implements Runnable
    {
         public String[] ergebnisT = null;
         public String in_quelltext = null;

         public ThreadAvrgccBegrenzt(String quelltext)
         {
             in_quelltext = quelltext;           
         }

         public void run()
         {
             ergebnisT=avrgccNormal(in_quelltext);
         }
    }

//*********************
//***** avrcpp ********
//*********************

    public final String[] avrcppNormal(String quelltext)
    {
        String[] erg = new String[2];

        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
     
        String name = "temporaer"+x;
   
        //Bitdarstellung einfach überall durch Dezimalzahlen ersetzen:
        //Also 0b00000000 durch 0, 0b00000001 durch 1 usw.:
        char[] bbb=new char[8];
        for(int qq=0;qq<256;qq++)
        {
            int ii=qq;
            for(int k=0;k<8;k++)
            {
                if(ii%2==1)
                    bbb[7-k]='1';
                else
                    bbb[7-k]='0';
                ii/=2;
            }

            String ersetze = "0b"+(new String(bbb));
            quelltext = parser.ersetze(quelltext,ersetze,""+qq);
        }

        //Quelltext vorverarbeiten:
        //Die Zeile #include<avr/io.h> wird ersetzt durch den Inhalt von kramann.avr.AVRIO:
        //Es wird davon ausgegangen, dass diese Include-Zeile die erste Zeile ist.
        String[] zeilen = parser.zerlegeZeilenNeu(quelltext);

        StringBuffer sb = new StringBuffer();

        sb.append("#include<stdio.h>");
        sb.append("\n");

        if(AVRIO.pdetemp==null)
            AVRIO.init();
        for(int ii=0;ii<AVRIO.pdetemp.length;ii++)
        {
            sb.append(AVRIO.pdetemp[ii]);
            sb.append("\n");
        }

        for(int ii=0;ii<zeilen.length;ii++)
        {
            if(zeilen[ii].trim().length()>="#include".length() 
            && zeilen[ii].trim().substring(0,"#include".length()).equals("#include"))
            { 
            }
            else 
            {
                sb.append(zeilen[ii]);
                sb.append("\n");
            }
        }

        quelltext = sb.toString();

        speichern(pfad+"/"+name+".cpp",quelltext);        

        erg[0] = callString("g++ -o "+pfad+"/"+name+" "+pfad+"/"+name+".cpp");
//        syst("rm -f "+pfad+"/"+name+".cpp");
        erg[1] = callString(pfad+"/"+name);
        syst("rm -f "+pfad+"/"+name);

        return erg;
    }

    public final String[] avrcpp(String quelltext)
    {
        //Normales Kompilieren und Ausführen:
        //return avrcppNormal(quelltext);

        //Zeitliche Begrenzung des Ausführens um die übergebene Millisekundenzeit:
        return avrcppBegrenzt(quelltext,5000);
    }

    public final String[] avrcppBegrenzt(String quelltext,long max_dauer_millis)
    {
        ThreadAvrcppBegrenzt tgb = new ThreadAvrcppBegrenzt(quelltext);
        try
        {
            Thread thread = new Thread(tgb);
            thread.start();
            long zeit=System.currentTimeMillis();
            while( System.currentTimeMillis()<zeit+max_dauer_millis && tgb.ergebnisT==null )
            {
                Thread.sleep(50);
            }

            if(tgb.ergebnisT!=null)
                return tgb.ergebnisT;
            else
                return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert.",null};
        }
        catch(Exception e)
        {
            return new String[] {"ERROR: Es wurde ein Endlosprozess implementiert und es traten Fehler beim Threadding auf.",null};
        }
    }    

    /**Hilfsklasse für avrcppBegrenzt, über die das Threadding realisiert wird.*/
    private final class ThreadAvrcppBegrenzt implements Runnable
    {
         public String[] ergebnisT = null;
         public String in_quelltext = null;

         public ThreadAvrcppBegrenzt(String quelltext)
         {
             in_quelltext = quelltext;           
         }

         public void run()
         {
             ergebnisT=avrcppNormal(in_quelltext);
         }
    }




    private final void speichern(String dateiname, String text)
    {
        try
        {
            File              meinFile = new File(dateiname);
            FileWriter   meinSchreiber = new FileWriter(meinFile);
            PrintWriter  meinPrinter   = new PrintWriter( meinSchreiber );            
            meinPrinter.print(text);            
            meinPrinter.close();   
        }
        catch(IOException e)
        {
            System.out.println(e.getMessage());
        }
    }

    public void speichernBinaer(String name, byte[] daten)
    {
        try
        {
            File meinFile = new File(name);
            FileOutputStream meinStream = new FileOutputStream(meinFile);            
            meinStream.write(daten);
            meinStream.close();
        }
        catch(IOException e)
        {
            System.out.println("Fehler beim Speichern:");
            System.out.println(e.getMessage());
        }    
    }

    private final String holeZufallspfad()
    {
        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
        
        pfad+="/.tmpz"+x;
        return pfad;
    }

    public final String encodeString(String text)
    {
        String datei = holeZufallspfad();
        //speichern(datei,text);

        byte[] bbb=new byte[]{};
        try
        {
            bbb=text.getBytes("UTF-8");
        }
        catch(Exception eeeeee)
        {
            System.out.println(eeeeee);
        }

        speichernBinaer(datei,bbb);
        //String ergebnis = callString("base64 -w 0 "+datei+" | encode ");
        byte[] ergebnis = callBytes("base64 -w 0 "+datei+" | encode ");
        syst("rm -f "+datei);
        String eee="";
        try
        {
            eee=new String(ergebnis,"UTF-8");
        }
        catch(Exception eeeee)
        {
            System.out.println(eeeee);
        }
        return eee;
    }

    public final String decodeString(String text)
    {
        String datei = holeZufallspfad();
        String datei2 = datei+"2";
        //speichern(datei,text);

        byte[] bbb=new byte[]{};
        try
        {
            bbb=text.getBytes("UTF-8");
        }
        catch(Exception eeeeee)
        {
            System.out.println(eeeeee);
        }

        speichernBinaer(datei,bbb);
        syst("cat "+datei+" | decode > "+datei2);
        syst("rm -f "+datei);
        byte[] ergebnis = callBytes("base64 -d -w 0 "+datei2);
        syst("rm -f "+datei2);
        String eee="";
        try
        {
            eee=new String(ergebnis,"UTF-8");
        }
        catch(Exception eeeee)
        {
            System.out.println(eeeee);
        }
        return eee;
    }

    public final String encodeBytes(byte[] bytes)
    {
        String datei = holeZufallspfad();
        speichernBinaer(datei,bytes);
        String ergebnis = callString("base64 -w 0 "+datei+" | encode ");
        syst("rm -f "+datei);
        return ergebnis;        
    }

    public final byte[] decodeBytes(String text)
    {
        String datei = holeZufallspfad();
        String datei2 = datei+"2";
        speichern(datei,text);
        syst("cat "+datei+" | decode > "+datei2);
        syst("rm -f "+datei);
        byte[] ergebnis = callBytes("base64 -d -w 0 "+datei2);
        syst("rm -f "+datei2);
        return ergebnis;
    }



    static
    {
        String pfad = kiba300.Kiba.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
System.out.println("Pfad="+pfad);
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);

        //Dynamische Bibliothek laden:
        System.load(pfad+"/aufrufen.so");
    }   

    public static void main(String[] args)
    {
        Linuxsystem ls = new Linuxsystem();

/*
        System.out.println("Testen der Funktion String aufruf(String):");
        System.out.println(ls.aufrufen("ls"));
        System.out.println("Testen der Funktion void syst(String):");
//        ls.syst("echo \"huhu\" | leafpad");
        ls.syst("echo \"huhu (Dieser Aufruf erfolgte mit system() in C)\"");
*/
        System.out.println("\n\nTesten von String call(String) mit \"javac huhu.java\":");
        String[] ergebnis = ls.call("javac huhu.java");
        for(int i=0;i<ergebnis.length;i++) 
            System.out.println(ergebnis[i]);

        System.out.println("\n\nTeste von encodeString und decodeString mit \"Dies ist ein Satz.\"");        
        String erg = ls.encodeString("Dies ist ein Satz.");
        System.out.println("encoded:"+erg);
        String decoded = ls.decodeString(erg);
        System.out.println("decoded:"+decoded);


        System.out.println("\n\nTeste von encodeBytes und decodeBytes mit {65,66,67}");        
        String er = ls.encodeBytes(new byte[] {65,66,67});
        System.out.println("encoded:"+er);
        byte[] de = ls.decodeBytes(er);
        System.out.println("decoded:"+(int)de[0]+","+(int)de[1]+","+(int)de[2]);

        System.out.println("PRÜFEN, OB DECODE/ENCODE AUCH MIT UMLAUTEN FUNKTIONIERT:");

        System.out.println("\n\nTeste von encodeString und decodeString mit \"Daß über mäandernden Mühlenrädern...\"");        
        erg = ls.encodeString("Daß über mäandernden Mühlenrädern...");
        System.out.println("encoded:"+erg);
        decoded = ls.decodeString(erg);
        System.out.println("decoded:"+decoded);

        System.out.println("gcc testen:");

        String quelltext = "#include<stdio.h>\n"
                          +"int main()\n"
                          +"{printf(\"huhu\");}\n";
        String[] erg2 = ls.gccNormal(quelltext);
        System.out.println("Fehlerausgabe des Kompilers:");
        System.out.println(erg2[0]);
        System.out.println("Ausgabe des Programms:");
        System.out.println(erg2[1]);

        System.out.println("gcc testen mit fehlerhaftem Quelltext:");

        quelltext = "#include<stdio.h>\n"
                          +"int main()\n"
                          +"{printf(\"huhu\")}\n";
        erg2 = ls.gccNormal(quelltext);
        System.out.println("Fehlerausgabe des Kompilers:");
        System.out.println(erg2[0]);
        System.out.println("Ausgabe des Programms:");
        System.out.println(erg2[1]);

        System.out.println("g++ testen mit Quelltext:");

        quelltext = "#include<iostream>\n"
                          +"using namespace std;\n"
                          +"int main()\n"
                          +"{cout<<\"huhu\"<<endl;}\n";
        erg2 = ls.gppNormal(quelltext);
        System.out.println("Fehlerausgabe des Kompilers:");
        System.out.println(erg2[0]);
        System.out.println("Ausgabe des Programms:");
        System.out.println(erg2[1]);
    }
}

